<?php

namespace App\Factory;

use App\Exception;

class PaymentFactory {

    private $connection;
    private $util;

    public function __construct (\Doctrine\DBAL\Connection $connection, 
                                 \App\Util\Util $util) {
        $this->connection = $connection;
        $this->util = $util;
    }
    
    public function getReceiptType($cdFilial, $receipt) {
        //===================== Busca por filial =====================
        $sql = "
            SELECT CDTIPORECE
              
              FROM TIPORECEINTELOCAL TP
    
              JOIN INTEGRACAO INTE 
                ON TP.CDINTEGRACAO   = INTE.CDINTEGRACAO
    
             WHERE INTE.IDINTEGRACAO = 'RAP' 
               AND CDFILIAL          = :CDFILIAL
               AND UPPER(REPLACE(CDTPRECEINTE, ' ', '')) = :CDTPRECEINTE    
        ";

        $params = [
            ":CDFILIAL"     => $cdFilial,
            ":CDTPRECEINTE" => strtoupper(preg_replace("/\s+/", "", $receipt))
        ];

        $result = $this->connection->fetchAll($sql, $params);
        
        if(!empty($result) && isset($result[0]["CDTIPORECE"])) {
            return $result[0]["CDTIPORECE"];
        }
        //===================== Busca por filial =====================

        $sql = "
            SELECT CDTIPORECE

              FROM TIPORECEINTE TP
            
              JOIN INTEGRACAO INTE 
                ON TP.CDINTEGRACAO   = INTE.CDINTEGRACAO

             WHERE INTE.IDINTEGRACAO = 'RAP' 
               AND UPPER(REPLACE(CDTPRECEINTE, ' ', '')) = :CDTPRECEINTE
        ";
        
        $params = [
            ":CDTPRECEINTE" => strtoupper(preg_replace("/\s+/", "", $receipt))
        ];
        
        $result = $this->connection->fetchAll($sql, $params);
        
        if(empty($result)) 
            throw new Exception\ParamsException("Não foi encontrado tipo de recebimento associado ao código: $receipt", 112);
        
        if(!isset($result[0]["CDTIPORECE"])) 
            throw new Exception\ParamsException("Não foi encontrado tipo de recebimento associado ao código: $receipt", 112);
        
        return $result[0]["CDTIPORECE"];
    }
    
    public function saveMovcaixadlv($cdfilial, $nrvendarest, $nrseqmovdlv, $paymentCode, $paymentValue, $cdcliente, $cdconsumidor) {
        $sql = "
            INSERT INTO MOVCAIXADLV
        		(CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
        		VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR)
        	VALUES
        		(:CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
        		:VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR)";

        $params = [
            "CDFILIAL"        => $cdfilial,
	        "NRVENDAREST"     => $nrvendarest,
	        "NRSEQMOVDLV"     => $nrseqmovdlv,
	        "IDTIPOMOVIVEDLV" => "E",
	        "CDCLIENTE"       => $cdcliente,
	        "CDCONSUMIDOR"    => $cdconsumidor,
	        "CDTIPORECE"      => $paymentCode,
	        "VRMOVIVENDDLV"   => $this->util->toFloat($paymentValue)
        ];

        return $this->connection->executeQuery($sql, $params);
    }
}
